from mayan.apps.dependencies.classes import PythonDependency

PythonDependency(
    module=__name__, name='Whoosh', version_string='==2.7.4'
)
PythonDependency(
    module=__name__, name='elasticsearch', version_string='==7.16.0'
)
PythonDependency(
    module=__name__, name='elasticsearch-dsl', version_string='==7.4.0'
)
