{% if not dashboard_skip_label %}
    <h4>
        {{ dashboard.label }}
    </h4>
    <hr>
{% endif %}

<div class="well">
    <div class="row">
        {% for widget in widgets %}
            {{ widget }}
        {% endfor %}
    </div>
</div>
